@@ -98,6 +98,7 @@ require 'checkObjPar.pl';
 require 'prLog.pl';
 require 'version.pl';
 require 'fileDir.pl';
+require 'dateTools.pl';
 
 
 
@@ -116,7 +117,8 @@ linkToDirs.pl - hard links files in directories with others
 
 	linkToDirs.pl [--linkWith copyBackupDir] [--linkWith ...]
 		      --targetDir targetForSourceDir
-		      [--progressReport number] [--dontLinkSymlinks]
+		      [--progressReport number[,timeframe]] 
+		       [--printDepth] [--dontLinkSymlinks]
 		      [--ignoreErrors] [--saveRAM] [-T tmpdir]
 		      [--createSparseFiles [--blockSize]]
 		      sourceDir ...
@@ -156,6 +158,14 @@ linked always in targetDir.
 =item B<--progressReport>, B<-P>
 
     print progress report after each 'number' files
+    additional you may add a time frame after which a message is
+    printed
+    if you want to print a report each 1000 files and after
+    one minute and 10 seconds, use: -P 1000,1m10s
+
+=item B<--printDepth>, B<-D>
+
+    print depth of actual read directory during backup
 
 =item B<--dontLinkSymlinks>
 
@@ -174,7 +184,6 @@ linked always in targetDir.
 
     directory for temporary files, default is </tmp>
 
-
 =item B<--createSparseFiles>, B<-s>
 
     if a file is indicated as a sparse file and that file has to be
@@ -224,8 +233,13 @@ my $CheckPar =
 				Option->new('-name' => 'progressReport',
 					    '-cl_option' => '--progressReport',
 					    '-cl_alias' => '-P',
-					    '-default' => 0,
-					    '-pattern' => '\A\d+\Z'),
+					    '-cf_key' => 'progressReport',
+					    '-default' => 0),
+				Option->new('-name' => 'printDepth',
+					    '-cl_option' => '--printDepth',
+					    '-cl_alias' => '-D',
+					    '-cf_key' => 'printDepth',
+					    '-cf_noOptSet' => ['yes', 'no']),
 				Option->new('-name' => 'dontLinkSymlinks',
 					    '-cl_option' => '--dontLinkSymlinks'),
 				Option->new('-name' => 'ignoreErrors',
@@ -256,6 +270,8 @@ my $linkWith = $CheckPar->getOptWithPar('linkWith');
 @$linkWith = () unless $linkWith;
 my $targetDir = $CheckPar->getOptWithPar('targetDir');
 my $progressReport = $CheckPar->getOptWithPar('progressReport');
+my $printDepth = $CheckPar->getOptWithoutPar('printDepth');
+$printDepth = $printDepth ? 'yes' : 'no';
 my $dontLinkSymlinks = $CheckPar->getOptWithoutPar('dontLinkSymlinks');
 my $ignoreErrors = $CheckPar->getOptWithoutPar('ignoreErrors');
 my (@sourceDir) = $CheckPar->getListPar();
@@ -275,6 +291,34 @@ my ($prLogKind) = ['A:BEGIN',
 		   'P:PROGRESS'];
 $prLog = printLog->new('-kind' => $prLogKind);
 
+# check progressReport settings
+my $progressDeltaTime = 0;
+if ($progressReport)
+{
+    my ($count, $t);
+    if ($progressReport =~ /,/)
+    {
+	($count, $t) = $progressReport =~ /\A(.*?),(.*)\Z/;
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["wrong format for option progressReport " .
+				 "time period <$t>"],
+		      '-exit' => 1)
+	    unless &dateTools::checkStr('-str' => $t);
+	$progressDeltaTime = &dateTools::strToSec('-str' => $t);
+    }
+    else
+    {
+	$count = $progressReport;
+    }
+    $prLog->print('-kind' => 'E',
+		  '-str' => ["counter <$count> for progress report " .
+			     "must be a positive integer"],
+		  '-exit' => 1)
+	unless $count =~ /\A\d+\Z/;
+    $progressReport = $count;
+}
+#print "progressReport=<$progressReport>, progressDeltaTime=<$progressDeltaTime>\n";
+
 my (@lt) = (@$linkWith);
 my (@lb) = (@sourceDir);
 my $lts = $targetDir;
@@ -436,14 +480,21 @@ if ($saveRAM)
     dbmopen(%md52file, $md52fileFile, 0600);
 }
 my $stat = Statistic->new('-prLog' => $prLog,
-			  '-progressReport' => $progressReport);
+			  '-progressReport' => $progressReport,
+			  '-progressDeltaTime' => $progressDeltaTime);
 
 
 #
 # read all linkWith directores
 #
 my $rrd = recursiveReadDir->new('-dirs' => $linkWith,
+				'-printDepth' => $printDepth,
+				'-printDepthPrlogKind' => 'P',
 				'-prLog' => $prLog);
+$prLog->print('-kind' => 'I',
+	      '-str' => ["start reading linkWith dirs <" .
+			 join('> <', @$linkWith) . '>'])
+    if @$linkWith;
 my ($f, $t);
 while (($f, $t) = $rrd->next())
 {
@@ -460,6 +511,13 @@ while (($f, $t) = $rrd->next())
 	next if exists $inode2md5{$devInode};  # already read
 
 	my $md5 = &::calcMD5($f, $prLog);
+	unless ($md5)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["could not calculate md5 sum of <$f>, skipping"]);
+	    next;
+	}
 	$stat->incrMd5();
 	my $md5size = "$md5-$size";
 	$inode2md5{$devInode} = $md5size;
@@ -479,6 +537,14 @@ while (($f, $t) = $rrd->next())
 	next if exists $inode2md5{$devInode};  # already read
 
 	my $md5 = &::readSymLinkCalcMd5($f, $prLog);
+	unless ($md5)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["could not calculate md5 sum of symlink <$f>," .
+			   " skipping"]);
+	    next;
+	}
 	$stat->incrMd5();
 	my $md5size = "L-$md5-$size";
 	$inode2md5{$devInode} = $md5size;
@@ -495,6 +561,8 @@ $targetDir = &::absolutePath($targetDir);
 #
 # copy / link @sourceDir
 #
+$prLog->print('-kind' => 'I',
+	      '-str' => ["start copying <" . join('> <', @sourceDir) . '>']);
 foreach my $d (@sourceDir)
 {
 #print "---$d---\n";
@@ -519,7 +587,9 @@ foreach my $d (@sourceDir)
 	$stat->incrDir();
     }
 
-    $rrd = recursiveReadDir->new('-dirs' => [$d],,
+    $rrd = recursiveReadDir->new('-dirs' => [$d],
+				 '-printDepth' => $printDepth,
+				 '-printDepthPrlogKind' => 'P',
 				 '-prLog' => $prLog);
     while (($f, $t) = $rrd->next())
     {
@@ -620,11 +690,26 @@ foreach my $d (@sourceDir)
 		if ($t eq 'f')
 		{
 		    my $md5 = &::calcMD5($f, $prLog);
+		    unless ($md5)
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' =>
+				      ["could not calculate md5 sum of <$f>, skipping"]);
+			next;
+		    }
 		    $md5size = "$md5-$size";
 		}
 		elsif ($t eq 'l')
 		{
 		    my $md5 = &::readSymLinkCalcMd5($f, $prLog);
+		    unless ($md5)
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' =>
+				      ["could not calculate md5 sum of symlink <$f>," .
+				       " skipping"]);
+			next;
+		    }
 		    $md5size = "L-$md5-$size";
 		}
 
@@ -842,7 +927,8 @@ sub new
     my $self = {};
 
     my (%params) = ('-prLog'             => undef,
-		    '-progressReport'    => undef
+		    '-progressReport'    => undef,
+		    '-progressDeltaTime' => 0
 		    );
 
     &::checkObjectParams(\%params, \@_, 'Statistic::new',
@@ -856,6 +942,9 @@ sub new
     $self->{'statRead'} = 0;
     $self->{'statMd5'} = 0;
 
+    $self->{'timeProgrReport'} =
+	($self->{'progressDeltaTime'} > 0) ? time : 0;
+
     bless $self, $class;
 }
 
@@ -909,13 +998,17 @@ sub checkProgressRead
     my $self = shift;
 
     my $n = $self->{'statRead'};
-    if ($self->{'progressReport'} and
-	$n % $self->{'progressReport'} == 0)
+    if (($self->{'progressReport'} and
+	$n % $self->{'progressReport'} == 0) or
+	($self->{'timeProgrReport'} > 0 and
+	 time >= $self->{'timeProgrReport'} + $self->{'progressDeltaTime'}))
     {
 	$self->{'prLog'}->print('-kind' => 'P',
 				'-str' =>
 				["read $n items, calced " .
 				$self->{'statMd5'} . " md5 sums"]);
+	$self->{'timeProgrReport'} = time
+	    if $self->{'timeProgrReport'} > 0;
     }
 }
 
@@ -926,8 +1019,10 @@ sub checkProgress
 
     my $n = $self->{'statCopy'} + $self->{'statLink'} +
 	$self->{'statDir'};
-    if ($self->{'progressReport'} and
-	$n % $self->{'progressReport'} == 0)
+    if (($self->{'progressReport'} and
+	 $n % $self->{'progressReport'} == 0) or
+	($self->{'timeProgrReport'} > 0 and
+	 time >= $self->{'timeProgrReport'} + $self->{'progressDeltaTime'}))
     {
 	$self->{'prLog'}->print('-kind' => 'P',
 				'-str' =>
@@ -935,6 +1030,8 @@ sub checkProgress
 				$self->{'statMd5'} . " md5 sums; " .
 				$self->{'statLink'} . " linked, " .
 				$self->{'statCopy'} . " copied"]);
+	$self->{'timeProgrReport'} = time
+	    if $self->{'timeProgrReport'} > 0;
     }
 }
 
