@@ -24,6 +24,7 @@ $main::STOREBACKUPVERSION = undef;
 
 use strict;
 use DB_File;           # Berkeley DB version 1
+use Digest::MD5 qw(md5_hex);
 
 
 sub libPath
@@ -113,7 +114,7 @@ storeBackupRecover.pl - recovers files saved with storeBackup.pl.
 
 	storeBackupRecover.pl -r restore [-b root] -t targetDir [--flat]
 		[-o] [--tmpdir] [--noHardLinks] [-p number] [-v] [-n]
-		[--cpIsGnu] [--noGnuCp]
+		[--cpIsGnu] [--noGnuCp] [-s]
 
 =head1 OPTIONS
 
@@ -169,6 +170,11 @@ storeBackupRecover.pl - recovers files saved with storeBackup.pl.
     installed
     (only relevant for sockets, block and character devices)
 
+=item B<--createSparseFiles>, B<-s>
+
+    creates sparse file from blocked files if full blockes
+    are filled with zeros
+
 =back
 
 =head1 COPYRIGHT
@@ -223,7 +229,10 @@ my $CheckPar =
 					    '-cl_option' => '-n',
 					    '-cl_alias' => '--noRestored'),
 				Option->new('-name' => 'noGnuCp',
-					    '-cl_option' => '--noGnuCp')
+					    '-cl_option' => '--noGnuCp'),
+				Option->new('-name' => 'createSparseFiles',
+					    '-cl_option' => '--createSparseFiles',
+					    '-cl_alias' => '-s')
 				]
 		    );
 
@@ -243,6 +252,7 @@ my $noRestoreParallel = $CheckPar->getOptWithPar('noRestoreParallel');
 my $verbose = $CheckPar->getOptWithoutPar('verbose');
 my $noRestored = $CheckPar->getOptWithoutPar('noRestored');
 my $noGnuCp = $CheckPar->getOptWithoutPar('noGnuCp');
+my $createSparseFiles = $CheckPar->getOptWithoutPar('createSparseFiles');
 
 
 my $prLog = printLog->new('-kind' => ['I:INFO', 'W:WARNING', 'E:ERROR',
@@ -693,8 +703,9 @@ while ((($md5sum, $compr, $devInode, $inodeBackup, $ctime, $mtime, $atime,
 		($old, $new) =
 		    $fork->add_block('-function' => \&uncompressCatBlock,
 				     '-funcPar' => ["$backupRoot/$filename",
+				     $backupRoot, $createSparseFiles,
 				     $targetFile, '\A\d.*', $uncompr, \@uncomprPar,
-				     $postfix, $prLog],
+				     $postfix, $size, $prLog],
 				     '-info' => [$targetFile, $uid, $gid, $mode,
 						 $atime, $mtime, $devInode, $comm]);
 		$prLog->print('-kind' => 'I',
@@ -873,30 +884,53 @@ sub setHardLinks
 sub uncompressCatBlock
 {
     my $fromDir = shift;
+    my $backupRoot = shift;
+    my $createSparseFiles = shift;
     my $toFile = shift;
     my $mask = shift;
     my $umcompr = shift;
     my $uncomprPar = shift;
     my $postfix = shift;
+    my $size = shift;
     my $prLog = shift;
 
-    local *DIR;
+    my $nBlocks = 0;
+    my (@entries, @md5, @compr);
+
+    my $fileIn =
+	pipeFromFork->new('-exec' => 'bzip2',
+			  '-param' => ['-d'],
+			  '-stdin' => "$fromDir/.md5BlockCheckSums.bz2",
+			  '-outRandom' => '/tmp/stbuPipeFrom11-',
+			  '-prLog' => $prLog);
+    my $l;
+    while ($l = $fileIn->read())
+    {
+	my ($l_md5, $l_compr, $l_f, $n);
+	chomp $l;
+	$n = ($l_md5, $l_compr, $l_f) = split(/\s/, $l, 3);
+	if ($n != 3)
+	{
+	    $prLog->print('-kind' => 'E',
+			  '-str' =>
+			  ["strange line in <$fromDir/.md5BlockCheckSums.bz2> " .
+			   "in line " . $fileIn->get('-what' => 'lineNr') .
+			   ":", "\t<$l>"]);
+	    return 1;
+	}
+	++$nBlocks;
+	push @md5, $l_md5;
+	push @compr, $l_compr;
+	push @entries, $l_f;
+    }
+    $fileIn->close();
+    $fileIn = undef;
+
+    &::createSparseFile($toFile, $size, $prLog) or exit 1;
+
+    my $md5null = '';
+
     my $ret = 0;
-    unless (opendir(DIR, $fromDir))
-    {
-	$prLog->print('-kind' => 'E',
-		      '-str' => ["cannot open <$fromDir>"]);
-	return 1;
-    }
-    my ($entry, @entries);
-    while ($entry = readdir DIR)
-    {
-	next unless $entry =~ /$mask/;
-
-	push @entries, $entry;
-    }
-    close(DIR);
-
     local *TO;
     unless (sysopen(TO, $toFile, O_CREAT | O_WRONLY))
     {
@@ -905,61 +939,78 @@ sub uncompressCatBlock
 	$ret = 1;
     }
 
-    foreach $entry (sort @entries)
+    my $blockSize = 0;
+    for (my $iBlock = 0 ; $iBlock < @entries ; ++$iBlock)
     {
+	my $entry = $entries[$iBlock];
+	my $md5 = $md5[$iBlock];
+	my $compr = $compr[$iBlock];
+	my $actSeekPos = $blockSize * $iBlock;
+
+	sysseek(TO, $actSeekPos, 0);
+
+	next if ($md5 eq $md5null);
+
 	my $buffer;
 	local *FROM;
 	my $fileIn = undef;
-	if ($entry =~ /$postfix\Z/)    # compressed block
+	if ($compr eq 'c')       # compressed block
 	{
 	    if ($main::IOCompressDirect)
 	    {
-		my $input = "$fromDir/$entry";
+		my $input = "$backupRoot/$entry";
 		my $uc = new IO::Uncompress::Bunzip2 $input;
 		while ($uc->read($buffer, 10*1024**2))
 		{
-		    unless (syswrite(TO, $buffer))
+		    my $n;
+		    unless ($n = syswrite(TO, $buffer))
 		    {
 			$prLog->print('-kind' => 'E',
 				      '-str' => ["writing to <$toFile> failed"]);
 			$ret = 1;
 		    }
+		    $blockSize += $n if $iBlock == 0;
 		}
-		next;
 	    }
-
-	    $fileIn =
-		pipeFromFork->new('-exec' => $uncompr,
-				  '-param' => \@uncomprPar,
-				  '-stdin' => "$fromDir/$entry",
-				  '-outRandom' => '/tmp/stbuPipeFrom11-',
-				  '-prLog' => $prLog);
-	    while ($fileIn->sysread(\$buffer, 10*1024**2))
+	    else
 	    {
-		unless (syswrite(TO, $buffer))
+		$fileIn =
+		    pipeFromFork->new('-exec' => $uncompr,
+				      '-param' => \@uncomprPar,
+				      '-stdin' => "$backupRoot/$entry",
+				      '-outRandom' => '/tmp/stbuPipeFrom11-',
+				      '-prLog' => $prLog);
+		while ($fileIn->sysread(\$buffer, 10*1024**2))
 		{
-		    $prLog->print('-kind' => 'E',
-				  '-str' => ["writing to <$toFile> failed"]);
-		    $ret = 1;
+		    my $n;
+		    unless ($n = syswrite(TO, $buffer))
+		    {
+			$prLog->print('-kind' => 'E',
+				      '-str' => ["writing to <$toFile> failed"]);
+			$ret = 1;
+		    }
+		    $blockSize += $n if $iBlock == 0;
 		}
 	    }
 	}
 	else           # block not compressed
 	{
-	    unless (sysopen(FROM, "$fromDir/$entry", O_RDONLY))
+	    unless (sysopen(FROM, "$backupRoot/$entry", O_RDONLY))
 	    {
 		$prLog->print('-kind' => 'E',
-			      '-str' => ["cannot read <$fromDir/$entry>"]);
+			      '-str' => ["cannot read <$backupRoot/$entry>"]);
 		return 1;
 	    }
 	    while (sysread(FROM, $buffer, 10*1024**2))
 	    {
-		unless (syswrite(TO, $buffer))
+		my $n;
+		unless ($n = syswrite(TO, $buffer))
 		{
 		    $prLog->print('-kind' => 'E',
 				  '-str' => ["writing to <$toFile> failed"]);
 		    $ret = 1;
 		}
+		$blockSize += $n if $iBlock == 0;
 	    }
 	}
 
@@ -972,7 +1023,27 @@ sub uncompressCatBlock
 	{
 	    close(FROM);
 	}
+
+	if ($iBlock == 0)
+	{
+	    $md5null = &::calcMD5null($blockSize)
+		if $createSparseFiles;
+	}
     }
     close(TO);
     return $ret;
 }
+
+
+############################################################
+# calc md5 sum of block with $size time zero
+sub calcMD5null
+{
+    my $size = shift;
+
+    my $null = pack('C', 0) x $size;
+
+    my $md5 = Digest::MD5->new();
+    $md5->add($null);
+    return $md5->hexdigest();
+}
