@@ -638,7 +638,7 @@ $prLog->print('-kind' => 'A',
 $prLog->print('-kind' => 'V',
 	      '-str' => ["storeBackupUpdateBackup.pl, $main::STOREBACKUPVERSION"]);
 
-::checkLockFile($lockFile, $prLog);
+&::checkLockFile($lockFile, $prLog);
 
 exit 1
     unless &deleteOldBackupDirs::checkTimeScaleFormat('archiveDurationDeltaCache',
