@@ -24,6 +24,7 @@ use Digest::MD5 qw(md5_hex);
 use Fcntl qw(O_RDWR O_CREAT);
 use Fcntl ':mode';
 use POSIX;
+use Cwd 'abs_path';
 
 require 'prLog.pl';
 require 'forkProc.pl';
@@ -244,76 +245,23 @@ sub splitFileDir
 sub absolutePath
 {
     my $dir = shift;
+    my $dirSave = $dir;
 
-    return undef unless $dir;
+    return undef unless $dir and -e $dir;
 
     # First look up in the hash table if that's already calculated
     return $main::absolutePathCache{$dir}
         if exists $main::absolutePathCache{$dir};
-    my $dirSave = $dir;
-
-    # if file itself is a hard link, follow the hard link(s) to something real
-    my $file = '';
-
-
-    $dir = $ENV{'PWD'} . "/$dir" unless ($dir =~ /^\//); # relative path
-
-    if (-e $dir)
-    {
-	$file = $dir;
-	while (-l $file)
-	{
-	    my $link = readlink($file);
-
-	    if (substr($link, 0, 1) ne "/")
-	    {
-		$file =~ s/[^\/]+$/$link/;
-	    }
-	    else
-	    {
-		$file = $link;
-	    }
-	}
-
-	($dir, $file) = &splitFileDir($file);
-	$file = "/$file";
-    }
-
-    my $ret = (-e "$dir$file") ? "$dir$file" : undef;
-    $ret =~ s/\/+/\//g;                # // -> /
-    while ($ret =~ s/\/\.\//\//g) {}   # /./ -> /
-    $ret =~ s/\A(.+?)\/+\Z/$1/;        # remove trailing /
-    $ret =~ s/\A\.\///;                # ./xxx -> xxx
-    $ret =~ s/\/\.\Z//;                # xxx/. -> xxx
-
-    # eliminate /../
-    my (@ret) = split(/\//, $ret);
-    if (@ret)
-    {
-	my (@rev) = reverse(@ret);
-	(@ret) = ();
-	my ($i, $j);
-	for ($i = $j = 0 ; $i < @rev ; $i++, $j++)
-	{
-	    if ($rev[$i] eq '..')
-	    {
-		$j++;
-	    }
-	    else
-	    {
-		push @ret, $rev[$j] if ($j < @rev);
-	    }
-	}
-	$ret = join('/', reverse @ret);
-    }
 
     if (@main::absolutePathCache > 100)
     {
 	my $del = shift @main::absolutePathCache;   # delete oldest
 	delete $main::absolutePathCache{$del};
     }
+    my $ret = abs_path($dir);
     push @main::absolutePathCache, $ret;
-    return $main::absolutePathCache{$dirSave} = $ret;
+    $main::absolutePathCache{$dirSave} = $ret;
+    return $ret;
 }
 
 
@@ -552,6 +500,9 @@ sub calcMD5
     local *FROM;
     unless (sysopen(FROM, $filename, O_RDONLY))
     {
+	$prLog->print('-kind' => 'E',
+		      '-str' => ["cannot open <$filename> in calcMD5"])
+	    if $prLog;
 	return undef;
     }
     my $md5 = Digest::MD5->new();
